/**
 * @author Vinay Chhabra
 **/
 
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.sql.*;

//Student entry form  
//  
class StudentForm extends JInternalFrame{
	
	//instance variables		
	private JTextField
		txtRollNumber = null,
		txtName = null,
	  txtCourse = null;

	// reference to the remote object.	
	//
	private	DatabaseInterface server = null;
	
	//constructor
	//
	public StudentForm(){
		
		//Create a JInternalFrame with the specified title 
		//and with resizability, closability, maximizability, 
		//and iconifiability specified. 
		//
		super("Student Form",true,true,false,true);
		
		//method used to create components 
		//
		setup();
		
		//Set the operation that will happen by default when 
		//the user initiates a "close" on this frame		
		//
		this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
		
		//Causes subcomponents of this JInternalFrame to be laid 
		//out at their preferred size.
	  //
		pack();
		
		//Move this component to a new location. 
    //
		setLocation(10,10);
		
		//Show this internal frame.
		//
		show();
		
	}  
	
	//Method used to create components and to 
	//get a remote reference.
	//
	private void setup(){
		
		//get a content pane object.
		//
		Container cp = getContentPane();
		
		//Panels for placing different components.
		JPanel topPanel,centerPanel,btnPanel;
		
		//create panels
		topPanel = SwingComp.createPanel(cp,BorderLayout.NORTH,new GridLayout(1,1));
		centerPanel = SwingComp.createPanel(cp,BorderLayout.CENTER,new GridLayout(3,2));
		
		//Create a new ButtonPanel object.
		//
		btnPanel = new ButtonPanel(new ButtonHandler());
		
		//Notify the layout manager to add 
		//the component to this container's layout using the 
		//specified constraints object.
		//
		cp.add(btnPanel,BorderLayout.SOUTH);
		
		//Set the border of this panel
		//
		centerPanel.setBorder(BorderFactory.createEmptyBorder(35,20,35,20));
    
    //{{ create labels, text boxes  }} //
    //
    JLabel lblTitle,lblRollNumber,lblName,lblCourse;
   
    lblTitle =  SwingComp.createLabel(topPanel,"STUDENT INFORMATION",JLabel.CENTER);

    lblRollNumber =  SwingComp.createLabel(centerPanel,"Roll Number");
    txtRollNumber =  SwingComp.createTextField(centerPanel,10);
   
    lblName =  SwingComp.createLabel(centerPanel,"Name");
    txtName =  SwingComp.createTextField(centerPanel,10);
      
	  lblCourse =  SwingComp.createLabel(centerPanel,"Course");
	  txtCourse = SwingComp.createTextField(centerPanel,10);
	 
	  // get a reference to the remote object.
    try{
   	   server = MDI.getServerObject();
   	  }catch(Exception ex){
   	  	System.out.println(ex.getMessage());
   	  }
         
	}
	
	
	//class to handle action events generated by the buttons.
	//
	private class ButtonHandler 
							implements ActionListener{
		
		//Invoked when the user clicks a button.
    //
		public void actionPerformed(ActionEvent ae){
			
			//Get the object on which the Event 
			//initially occurred.
      //
			JButton temp = (JButton)ae.getSource();
			
			//Get the action command for this button.
			String actionCommand = temp.getActionCommand();
			
			try{
			
			//if add button is clicked call the add method
			//
			if(actionCommand.equals("Add"))
				 add();
			
			//if Update button is clicked call the update method
	    //
			else if(actionCommand.equals("Update"))	 
			   update();
			
			//if Delete button is pressed call the delete method
	    //
			else if(actionCommand.equals("Delete"))	 
			   delete();
			
			//if Clear button is clicked call the clear method
	    //					    
			else if(actionCommand.equals("Clear"))	 
			   clear();
			
			//if Find button is clicked call the find method
	    //
			else if(actionCommand.equals("Find"))	 
			   find();
			      
			}catch(Exception ex){
				showMessageDialog(ex.getMessage());
		     
			}
		}
		
		//method invoked when add button is pressed
		private void add()
			throws RemoteException,SQLException{
								
			//Check whether the form is properly filled or not.
			// 
			if(check()){
				showMessageDialog("Please fill complete information.");
				return;
			}
									
			//Parse the string argument.
			//
			int rollno = Integer.parseInt(txtRollNumber.getText());
			
			//Get the name of student
			//
			String name = txtName.getText();
			
			//Get the course
			//
			String course = txtCourse.getText();
      
      //create a student object with specified rollno,
      //name and course.
      //
      Student std = new Student(rollno,name,course);                                      
      
      //Call the remote method to add a new student record
      //
      String res = server.addRecord(std);
      
      //Clear all text boxes
      //
      clear();		
      
      //Show the message returned by the server.
      //
      showMessageDialog(res);
	  }
	  
	  //method invoked when update button is pressed
    //
	  private void update()
			throws RemoteException,SQLException{
			
			//Check whether the form is properly filled or not.
			// 
			if(check()){
				showMessageDialog("Please fill complete information.");
				return;
			}
						
			//get the roll number
			//
			int rollno = Integer.parseInt(txtRollNumber.getText());
			
			//get the name
			//
			String name = txtName.getText();
			
			//get the course
			//
			String course = txtCourse.getText();
			
			//create a student object with specified rollno,
      //name and course.
      //      
      Student std = new Student(rollno,name,course);
      
      //call the remote method to update 
      //the record.
      //
      String res = server.updateRecord(std);		
      
      //clear values in the text boxes
      //
      clear();
      
      //Show the message returned by the server.
      //
      showMessageDialog(res);

	  }
	  
	  //Method invoked when Delete button is pressed
	  //
		private void delete()
			throws RemoteException,SQLException{
			
		 //get the roll number
		 //
		 String value = txtRollNumber.getText().trim();	 		 	 	
     
	 	 if(value.equals("")){
	 	 	 
	 	 	//if roll number is not entered
	 	 	//
			showMessageDialog("Please enter the Roll number.");
			return;
		 }
		 
		 //Parse the string argument.
		 //
		 int rollno = Integer.parseInt(txtRollNumber.getText());
		  		
		 //Call the remote method to delete the record.
		 //      
     String res = server.deleteRecord(rollno);		
     
     //Clear the text boxes.
     //
     clear();
     
     //Show message returned by the server.
     //
     showMessageDialog(res);
      
	  }
	  
	 
	 //method invoked when Clear button is pressed
	 //
	 private void clear(){
	 	
	 	//set the text of all the text boxes.
	 	//
	 	txtRollNumber.setText("");
	 	txtName.setText("");
	 	txtCourse.setText("");
	 }
	
	 //method used to find a record
   private void find()
	 	 throws RemoteException, SQLException{
	 	
	 	//Get the Roll number of student.
	 	//
	 	String value = txtRollNumber.getText().trim();	 		 	 	
	 	
	 	if(value.equals("")){
	 		
	 		//if roll number is not entered
	 	 	//
			showMessageDialog("Please enter the Roll number.");
			return;
		}
		
	 	int rollno = Integer.parseInt(txtRollNumber.getText());
	 	
	 	// Call the remote method to search the record.
	 	//
	 	Object res = server.findRecord(rollno);
    
    //If a string is returned by server.
    //    
    if(res instanceof String){
    	String temp = (String)res;
    	showMessageDialog(temp);
    	clear();
    }
    else{
    	 Student temp = (Student)res;
    	 
    	 //method to fill text boxes with values.
    	 //
    	 fill(temp);
      }
	 }
	 
	 //Check whether values are filled or not,
	 //
	 private boolean check(){
	 	boolean value = txtRollNumber.getText().trim().equals("") ||
	 		      txtName.getText().trim().equals("")||
	 		      txtCourse.getText().trim().equals("");	
	 	        
	 	return value;	              
	 }
	
	 //fill the text boxes.
	 //
	 private void fill(Student std){
    
    txtName.setText(std.getName());
	 	txtCourse.setText(std.getCourse());
	 }
	 
	 //method used to show a dialog box.
	 //	
	 private void showMessageDialog(String msg){
	 	 		JOptionPane.showMessageDialog(null,msg
		     	,"INFORMATION",JOptionPane.INFORMATION_MESSAGE);
	 }
	 
	}
}
			
 