/**
 * @author Vinay Chhabra
 */

import java.io.*;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;

//server class
public class RegisterServerImpl extends UnicastRemoteObject 
								implements RegisterInterface {
		
		//constructor							
		public RegisterServerImpl()throws RemoteException{
			System.out.println("Creating Server Object");
		}
		
		//method invoked by clients to register.
		//
		public String register(String name)
				throws RemoteException,IOException {
			
			//Constructs a FileWriter object given a 
			//file name with a boolean indicating whether 
			//or not to append the data written.
			//
			FileWriter file = new FileWriter("regfile.txt",true);
			
			//Creating a buffered character-output stream that 
			//uses a default-sized output buffer.
			//
			BufferedWriter buf = new BufferedWriter(file);
			
			//Write the name of the user.
      //
			buf.write(name);
			
			//Write a line separator
			//
			buf.newLine();
			
			//Flush the stream.
      //
			buf.flush();
			
			//Close the stream.
      //
			buf.close();
			
			//Close the stream.
      //
      file.close();
      return name +" registered successfully"; 
		}
		
		//method invoked remotely to get the
		//list of already registered users
		//
		public String getList()
			throws RemoteException,IOException {
      
      //Create a new FileReader object
      //
      FileReader file = new FileReader("regFile.txt");
      
      //Create a buffering character-input stream 
      //that uses a default-sized input buffer.
			//
      BufferedReader buf = new BufferedReader(file);
      String list = "";
      String temp = "";
      do{
         //Read a line of text. A line is considered 
         //to be terminated by any one of a line 
         //feed ('\n'), a carriage return ('\r'), 
         //or a carriage return followed immediately 
         //by a linefeed.
         // 
         temp = buf.readLine();
         if(temp!=null)
         	  list+=temp+"\n";
       }while(temp!=null);
       	
       return list;	 	  
		}
		
		//main method
		public static void main(String [] args) {
		
		try{
			 //create a server object	
		 	 RegisterServerImpl regServer = new RegisterServerImpl();
		 	 
		 	 //Create and export a Registry on the 
		 	 //local host that accepts requests on 
		 	 //port number 2000.
		 	 //
			 Registry reg = LocateRegistry.createRegistry(2000);
			 
			 //Rebind the specified name to a 
			 //remote object. 
			 //
			 reg.rebind("Server",regServer);
			 System.out.println("Server is ready");
	 	  }catch(RemoteException ex){
	 		   System.out.println("Error :"+ex.getMessage());
	 	  }
	 }
}